#define F_CPU 16000000
#define USART_BAUDRATE 31250
#define BAUD_PRESCALE (((F_CPU / (USART_BAUDRATE * 16UL))) - 1)

// Sound-banks available on the VS1053.
// 0x00 : GM1 (default)
// 0x78 : Drums
// 0x79 : GM2
#define SOUND_GM1 0x00
#define SOUND_DRUMS 0x78
#define SOUND_GM2 0x79

#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <stdio.h>

#define NUM_OF_SAMPLES  1
#define CAP_THRESHOLD   50
#define NUM_KEYS 8

int t0;
int instruments[] = {8, 55, 10, 120, 5, 100, 6, 7, 9, 46, 127, 106, 104, 119, 121};
int instrument;
int nr;

void USART0_init() {
    /* setează baud rate */
    UBRR0H = 0;
    UBRR0L = 31;
 
    /* pornește transmițătorul */
    UCSR0B = (1<<TXEN0);
 
    /* setează formatul frame-ului: 8 biți de date, 1 bit de stop, fără paritate */
    UCSR0C = (3<<UCSZ00);
}

void USART0_transmit(char data)
{
    /* așteaptă până când buffer-ul e gol */
    while(!(UCSR0A & (1<<UDRE0)));
 
    /* pune datele în buffer; transmisia va porni automat în urma scrierii */
    UDR0 = data;
}

void sendMIDI(char data) {
  USART0_transmit(data);
}

// Comunicatia dintre shield si placuta pm
void talkMIDI(char cmd, char data1, char data2) {
  sendMIDI(cmd);
  sendMIDI(data1);
  if((cmd & 0xF0) <= 0xB0) {
  		sendMIDI(data2);
  }
}

void init() {
	// initializarea celor 8 butoane
	DDRA = 0;
	PORTA = 0xff;

	//initializare buton next
	DDRC = 0;
	PORTC = 0xff;

	// setare RST shield
	DDRB |= (1<< PB1);

	PORTB &= ~(1 << PB1);
	_delay_ms(100);

	PORTB |= (1 << PB1);
	_delay_ms(100);
}

// functie pentru redare nota
void noteOn(char channel, char note, char attack_velocity) {
	talkMIDI( (0x90 | channel), note, attack_velocity);
}

// functie pentru oprire redare nota
void noteOff(char channel, char note, char release_velocity) {
  talkMIDI( (0x80 | channel), note, release_velocity);
}

// main-ul programului
int main(void) {
	t0 = 0;
	instrument = 8;
	init();
	USART0_init();

	talkMIDI(0xB0, instrument, 127); 
   	talkMIDI(0xC0, instrument, 0); 

	while(1) {
		// selectare instrument urmator
		if ((PINC & (1 << PC0)) == 0) {
			nr++;
			nr = nr % 15;
			instrument = instruments[nr];
			_delay_ms(10);
		}
		//----------------------------------------
		if(((PINA & (1 << PA0)) == 0) && (t0 == 0)){
			noteOn(0,86,127);
			t0 = 1;
			_delay_ms(10);
		}
		else if(((PINA & (1 << PA0)) == 1) && (t0 == 1)){
			noteOff(0,86,0);
			t0 = 0;
			_delay_ms(10);
		}
		//----------------------------------------
		if(((PINA & (1 << PA1)) == 0)){
			noteOn(0,84,127);
			_delay_ms(10);
		}
		else if(((PINA & (1 << PA1)) == 1)){
			noteOff(0,84,0);
			_delay_ms(10);
		}
		//----------------------------------------
		if(((PINA & (1 << PA2)) == 0) ){
			noteOn(0,80,127);
			_delay_ms(10);
		}
		else if(((PINA & (1 << PA2)) == 1)){
			noteOff(0,80,0);
			_delay_ms(10);
		}
		//----------------------------------------
		if(((PINA & (1 << PA3)) == 0)){
			noteOn(0,78,127);
			_delay_ms(10);
		}
		else if(((PINA & (1 << PA3)) == 1)){
			noteOff(0,78,0);
			_delay_ms(10);
		}
		//--------------------------------------
		if(((PINA & (1 << PA4)) == 0) ){
			noteOn(0,74,127);
			_delay_ms(10);
		}
		else if(((PINA & (1 << PA4)) == 1)){
			noteOff(0,74,0);
			_delay_ms(10);
		}
		//----------------------------------------
		if(((PINA & (1 << PA5)) == 0) ){
			noteOn(0,72,127);
			_delay_ms(10);
		}
		else if(((PINA & (1 << PA5)) == 1)){
			noteOff(0,72,0);
			_delay_ms(10);
		}
		//----------------------------------------
		if(((PINA & (1 << PA6)) == 0) ){
			noteOn(0,70,127);
			_delay_ms(10);
		}
		else if(((PINA & (1 << PA6)) == 1)){
			noteOff(0,80,0);
			_delay_ms(10);
		}
		//----------------------------------------
		if(((PINA & (1 << PA7)) == 0) ){
			noteOn(0,68,127);
			_delay_ms(10);
		}
		else if(((PINA & (1 << PA7)) == 1)){
			noteOff(0,68,0);
			_delay_ms(10);
		}
	}
}
